<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <!-- ******************************************* -->
    <!-- Centre de Calcul de l'IN2P3 - Lyon (France) -->
    <!--             http://cc.in2p3.fr/             -->
    <!-- ******************************************* -->
    <!-- author: Jonathan Moutarde                   -->
    <!-- ******************************************* -->
    <xsl:output method="html" indent="yes"/>

    <xsl:variable name="debug" select="false()" />

    <xsl:template match="/">
        <xsl:if test="$debug">
            <xsl:message terminate="no">Debug from ChartRenderer.xsl
    XML <xsl:copy-of select="/" /></xsl:message>
        </xsl:if>
        <html>
            <head>
                <xsl:if test="/supermap/meta/property[@name='title']">
                    <title><xsl:value-of select="//meta/property[@name='title']/text()"/></title>
                </xsl:if>
                <link rel="stylesheet" href="/resource/css/chart.css"/>

                <script src="/resource/js/webcomponentsjs/webcomponents.js" type="text/javascript"></script>
                <link rel="import" href="/resource/html/polymer/highchart-element/polymer-highchart.html"/>
            </head>
            <body>
                <xsl:for-each select="/supermap/map">
                    <polymer-highchart>
                        <xsl:attribute name="chart-config">
                            <xsl:call-template name="open" />
                            <xsl:apply-templates select="./*[1]"> <xsl:with-param name="comma" /> </xsl:apply-templates>
                            <xsl:apply-templates select="./*[position()>1]" />
                            <xsl:call-template name="close" />
                        </xsl:attribute>
                    </polymer-highchart>
                </xsl:for-each>
            </body>
        </html>
    </xsl:template>

    <xsl:template match="property[@type='object']">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text" select="./@name" /> </xsl:call-template>
        <xsl:call-template name="open" > <xsl:with-param name="before" select="':'" /> </xsl:call-template>
        <xsl:apply-templates select="./*[1]"> <xsl:with-param name="comma" /> </xsl:apply-templates>
        <xsl:apply-templates select="./*[position()>1]" />
        <xsl:call-template name="close" />
    </xsl:template>
    <xsl:template match="property[@type='array']">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text" select="./@name" /> </xsl:call-template>
        <xsl:call-template name="open" > <xsl:with-param name="before" select="':'" /> <xsl:with-param name="opener" select="'['" /> </xsl:call-template>
        <xsl:apply-templates select="./*[1]"> <xsl:with-param name="comma" /> </xsl:apply-templates>
        <xsl:apply-templates select="./*[position()>1]" />
        <xsl:call-template name="close"> <xsl:with-param name="closer" select="']'" /> </xsl:call-template>
    </xsl:template>
    <xsl:template match="property[@type='boolean']">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text" select="./@name" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="':'" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
    </xsl:template>
    <xsl:template match="property[@type='number']">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text" select="./@name" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="':'" /> </xsl:call-template>
        <xsl:choose>
            <xsl:when test="text()='NaN'">
                <xsl:call-template name="write-json"> <xsl:with-param name="text" select="'Number__NaN'" /> </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="property[@type='string']">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text" select="./@name" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="':'" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text">
            <xsl:call-template name="string-replace-all"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
        </xsl:with-param> </xsl:call-template>
    </xsl:template>
    <xsl:template match="property">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text" select="./@name" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="':'" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
    </xsl:template>
    <xsl:template match="object">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="open" />
        <xsl:apply-templates select="./*[1]"> <xsl:with-param name="comma" /> </xsl:apply-templates>
        <xsl:apply-templates select="./*[position()>1]" />
        <xsl:call-template name="close" />
    </xsl:template>
    <xsl:template match="array">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="open" > <xsl:with-param name="opener" select="'['" /> </xsl:call-template>
        <xsl:apply-templates select="./*[1]"> <xsl:with-param name="comma" /> </xsl:apply-templates>
        <xsl:apply-templates select="./*[position()>1]" />
        <xsl:call-template name="close"> <xsl:with-param name="closer" select="']'" /> </xsl:call-template>
    </xsl:template>
    <xsl:template match="number">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:choose>
            <xsl:when test="text()='NaN'">
                <xsl:call-template name="write-json"> <xsl:with-param name="text" select="'Number__NaN'" /> </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="boolean">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
    </xsl:template>
    <xsl:template match="string">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="$comma" /> </xsl:call-template>
        <xsl:call-template name="write-json"> <xsl:with-param name="text">
            <xsl:call-template name="string-replace-all"> <xsl:with-param name="text" select="./text()" /> </xsl:call-template>
        </xsl:with-param> </xsl:call-template>
    </xsl:template>
    <xsl:template match="value">
        <xsl:param name="comma" select="','" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="concat($comma, ./text())" /> </xsl:call-template>
    </xsl:template>


    <!-- Utilities -->
    <xsl:template name="string-replace-all">
    <!-- Replace all " by ' in a string -->
        <xsl:param name="text" />
        <xsl:param name="replace">"</xsl:param>
        <xsl:param name="by">'</xsl:param>
        <xsl:choose>
            <xsl:when test="contains($text, $replace)">
                <xsl:value-of select="substring-before($text,$replace)"/>
                <xsl:value-of select="$by"/>
                <xsl:call-template name="string-replace-all">
                    <xsl:with-param name="text" select="substring-after($text,$replace)"/>
                    <xsl:with-param name="replace" select="$replace"/>
                    <xsl:with-param name="by" select="$by"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$text"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="raw-text">
    <!-- Write a text without spaces (before and after) -->
        <xsl:param name="text"/>
        <xsl:text disable-output-escaping="yes"></xsl:text><xsl:value-of select="$text"/><xsl:text disable-output-escaping="yes"></xsl:text>
    </xsl:template>
    <xsl:template name="write-json">
    <!-- Write a string escaping the " character for avoiding conflict with the HTML attribute (Necessary to create a string JSON like in a HTML attribute -->
        <xsl:param name="text"/>
        <xsl:text disable-output-escaping="yes">&quot;</xsl:text><xsl:value-of select="$text"/><xsl:text disable-output-escaping="yes">&quot;</xsl:text>
    </xsl:template>

    <!-- Utilities to use for opening and closing JSON objects -->
    <xsl:template name="open">
        <xsl:param name="before" select="''" />
        <xsl:param name="opener" select="'{'" />
        <xsl:param name="after" select="''" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="concat($before, $opener, $after)" /> </xsl:call-template>
    </xsl:template>
    <xsl:template name="close">
        <xsl:param name="before" select="''" />
        <xsl:param name="closer" select="'}'" />
        <xsl:param name="after" select="''" />
        <xsl:call-template name="raw-text"> <xsl:with-param name="text" select="concat($before, $closer, $after)" /> </xsl:call-template>
    </xsl:template>
</xsl:stylesheet>